from pyopencell.client import Client
from pyopencell.resources.base_resource import BaseResource
from pyopencell.responses.response import ActionStatus


class Invoice(BaseResource):
    _name = "invoice"
    _url_path = "/invoice"

    invoiceId = 0
    invoiceType = ""
    billingAccountCode = ""
    sellerCode = ""
    subscriptionCode = ""
    orderNumber = ""
    dueDate = 0
    invoiceDate = 0
    categoryInvoiceAgregate = [{}]
    taxAggregate = [{}]
    invoiceNumber = ""
    amountWithoutTax = 0
    amountTax = 0
    amountWithTax = 0
    paymentMethod = "DIRECTDEBIT"
    pdfFilename = ""
    pdf = ""
    netToPay = 0
    invoiceMode = "DETAILLED"
    dueBalance = 0
    isDraft = True

    @classmethod
    def sendByEmail(cls, invoice_id):
        """
        Send the invoice by email

        :return:
        """
        response_data = Client().post(
            "{}/{}".format(cls._url_path, "sendByEmail"),
            {
                "invoiceId": invoice_id,
            })
        return ActionStatus(**response_data)




class PyOpenCellHTTPException(Exception):
    message = "Network problem accessing OpenCell API. Exception: \n {}"

    def __init__(self, error_msg):
        self.message = self.message.format(error_msg)
        super(PyOpenCellHTTPException, self).__init__(self.message)


class PyOpenCellAPIException(Exception):
    message = """OpenCell API returned an error response. \n
    Request: {verb} {url} \n
    Response: status={status} body={body}
    """

    def __init__(self, verb, url, status, body):
        self.message = self.message.format(verb=verb, url=url, status=status, body=body)
        self.api_error = body
        super(PyOpenCellAPIException, self).__init__(self.message)



from pyopencell.client import Client
from pyopencell.resources.base_resource import BaseResource
from pyopencell.responses.response import Response, ActionStatus


class CustomerAccount(BaseResource):
    _name = "customerAccount"
    _url_path = "/account/customerAccount"

    auditable = {
        "created": None,
        "updated": None,
        "creator": "",
        "updater": "",
    }
    id = None
    code = ""
    externalRef1 = ""
    externalRef2 = ""
    description = ""
    name = {
        "title": "",
        "firstName": "",
        "lastName": "",
    }
    jobTitle = ""
    updatedCode = ""
    businessAccountModel = {
        "auditable": {
            "created": None,
            "updated": None,
            "creator": "",
            "updater": "",
        },
        "id": None,
        "code": "",
        "description": "",
        "updatedCode": "",
    }
    contactInformation = {
        "email": "",
        "phone": "",
        "mobile": "",
        "fax": "",
    }
    vatNo = ""
    language = ""
    # TODO  Create this resource methodPayment
    methodOfPayment = {
        "paymentMethodType": "CARD",
        "id": 12345,
        "disabled": True,
        "alias": "...",
        "preferred": True,
        "customerAccountCode": "...",
        "info1": "...",
        "info2": "...",
        "info3": "...",
        "info4": "...",
        "info5": "...",
        "bankCoordinates": {
            "bankCode": "...",
            "branchCode": "...",
            "accountNumber": "...",
            "key": "...",
            "iban": "...",
            "bic": "...",
            "accountOwner": "...",
            "bankName": "...",
            "bankId": "...",
            "issuerNumber": "...",
            "issuerName": "...",
            "ics": "..."
        },
        "mandateIdentification": "...",
        "mandateDate": 12345,
        "cardType": "CB",
        "owner": "...",
        "monthExpiration": 12345,
        "yearExpiration": 12345,
        "tokenId": "...",
        "cardNumber": "...",
        "issueNumber": "...",
        "userId": "..."
    }
    customFields = []
    additionalDetails = {
        "companyName": "",
        "positon": "",
        "instantMessengers": "",
    }
    registrationNo = ""
    billingAccounts = {
        "billingAccount": []
    }
    status = ""

    @classmethod
    def get(cls, customerAccountCode):
        """
        Returns a customer instance obtained by id.

        :param id:
        :return: CustomerAccount:
        """
        response_data = Client().get(cls._url_path, customerAccountCode=customerAccountCode)

        status = response_data.get("status")
        if status and status != "SUCCESS":
            return ActionStatus(**response_data)

        return Response(cls, **response_data)



from pyopencell.client import Client
from pyopencell.resources.base_resource import BaseResource
from pyopencell.responses.response import Response
from pyopencell.responses.action_status import ActionStatus


class Subscription(BaseResource):
    _name = "subscription"
    _url_path = "/billing/subscription"
    _default_termination_reason = "CC_TERMINATION"

    auditable = {
        "created": None,
        "updated": None,
        "creator": "",
        "updater": "",
    }
    code = ""
    description = ""
    userAccount = ""
    updatedCode = ""
    offerTemplate = ""
    subscriptionDate = None
    terminationDate = None
    endAgreementDate = None
    customFields = []
    terminationReason = ""
    orderNumber = ""
    minimumAmountEl = ""
    minimumAmountElSpark = ""
    minimumLabelEl = ""
    minimumLabelElSpark = ""
    subscribedTillDate = None
    renewed = None
    renewalNotifiedDate = None
    renewalRule = {
        "initialyActiveFor": None,
        "initialyActiveForUnit": "",
        "autoRenew": None,
        "daysNotifyRenewal": None,
        "endOfTermAction": "",
        "terminationReasonCode": "",
        "renewFor": None,
        "renewForUnit": "",
        "extendAgreementPeriodToSubscribedTillDate": None
    }
    billingCycle = "",
    seller = "",
    autoEndOfEngagement = None,
    ratingGroup = ""

    @classmethod
    def get(cls, subscription_code):
        """
        Returns a subscription instance obtained by code.

        :param subscriptionCode:
        :return: Subscription:
        """
        response_data = Client().get(
            cls._url_path,
            subscriptionCode=subscription_code
        )

        status = response_data.get("status")
        if status and status != "SUCCESS":
            return ActionStatus(**response_data)

        return Response(cls, **response_data)

    @classmethod
    def create(cls, **kwargs):
        """
        Creates a subscription instance.

        :param kwargs:
        :return:
        """
        response_data = Client().post(cls._url_path, kwargs)

        return ActionStatus(**response_data)

    def activate(self, services_to_activate):
        """
        Activate services. Subscription should not be in status (RESILIATED OR CANCELLED).

        :param services_to_activate:
        :return:
        """
        action = "activateServices"
        kwargs = {
            "subscription": self.code,
            "servicesToActivate": {
                "service": services_to_activate,
            }
        }
        response_data = Client().post(
            "{}/{}".format(self._url_path, action),
            kwargs)

        return ActionStatus(**response_data)

    def terminate(self, termination_date):
        """
        Terminate OpenCell's Subscription.

        :param termination_date:
        :return:
        """
        response_data = Client().post(
            "{}/{}".format(self._url_path, "terminate"),
            {
                "subscriptionCode": self.code,
                "terminationDate": termination_date,
                "terminationReason": self._default_termination_reason,
            })
        return ActionStatus(**response_data)

    def terminateServices(self, termination_date, services):
        """
        Terminate OpenCell's Subscription services

        :param terminationDate:, services:
        :return:
        """
        response_data = Client().post(
            "{}/{}".format(self._url_path, "terminateServices"),
            {
                "subscriptionCode": self.code,
                "terminationDate": termination_date,
                "services": services,
                "terminationReason": self._default_termination_reason,
            })
        return ActionStatus(**response_data)

    def applyOneShotCharge(self, one_shot_charge_code):
        """
        Apply one shot charge to the OpenCell's Subscription.

        :param one_shot_charge_code:
        :return:
        """
        response_data = Client().post(
            "{}/{}".format(self._url_path, "applyOneShotChargeInstance"),
            {
                "subscription": self.code,
                "oneShotCharge": one_shot_charge_code,
            })
        return ActionStatus(**response_data)



from pyopencell.responses.action_status import ActionStatus
from pyopencell.responses.paging import Paging


class PagedResponse():
    paging = {}
    action_status = {}

    def __init__(self, cls, **response_data):
        self.paging = Paging(**response_data.get("paging"))
        self.action_status = ActionStatus(**response_data.get("actionStatus"))

        instances = []

        instances_from_response = cls.get_instances_from_response(**response_data)

        for instance in instances_from_response:
            instances.append(cls.items_resource_class(**instance))

        setattr(self, cls._name, instances)



from pyopencell.client import Client
from pyopencell.resources.base_resource import BaseResource
from pyopencell.responses.response import Response, ActionStatus


class Customer(BaseResource):
    _name = "customer"
    _url_path = "/account/customer"

    auditable = {
        "created": None,
        "updated": None,
        "creator": "",
        "updater": "",
    }
    id = None
    code = ""
    externalRef1 = ""
    externalRef2 = ""
    description = ""
    customerCategory = ""
    name = {
        "title": "",
        "firstName": "",
        "lastName": "",
    }
    updatedCode = ""
    customerBrand = ""
    jobTitle = ""
    seller = ""
    businessAccountModel = {
        "auditable": {
            "created": None,
            "updated": None,
            "creator": "",
            "updater": "",
        },
        "id": None,
        "code": "",
        "description": "",
        "updatedCode": "",
    }
    mandateIdentification = ""
    customFields = []
    mandateDate = None
    vatNo = ""
    additionalDetails = {
        "companyName": "",
        "positon": "",
        "instantMessengers": "",
    }
    registrationNo = ""
    contactInformation = {
        "email": "",
        "phone": "",
        "mobile": "",
        "fax": "",
    }
    customerAccounts = {
        "customerAccount": []
    }

    @classmethod
    def get(cls, customerCode):
        """
        Returns a customer instance obtained by id.

        :param id:
        :return: Customer:
        """
        response_data = Client().get(cls._url_path, customerCode=customerCode)

        status = response_data.get("status")
        if status and status != "SUCCESS":
            return ActionStatus(**response_data)

        return Response(cls, **response_data)

    @classmethod
    def create(cls, **kwargs):
        """
        Creates a customer instance.

        :param kwargs:
        :return:
        """
        response_data = Client().post(cls._url_path, kwargs)

        return ActionStatus(**response_data)



class BaseResourceList:
    _name = ""
    _url_path = ""
    items_resource_class = None
    items_per_page = 10

    def __init__(self, kwargs_list):
        for base_resource in kwargs_list:
            for kwarg, value in base_resource.items():
                setattr(self, kwarg, value)

    @classmethod
    def get_instances_from_response(self, **response):
        return response[self._name]



import logging
import requests
import json

from pyopencell import helpers
from pyopencell import exceptions


logger = logging.getLogger(__name__)


class Client(object):
    """ PyOpenCell class
    """

    def __init__(self):
        self.baseurl = helpers.getenv_or_fail('OPENCELL_BASEURL')
        self.username = helpers.getenv_or_fail('OPENCELL_USER')
        self.password = helpers.getenv_or_fail('OPENCELL_PASSWORD')

    def get(self, route, **kwargs):
        """ Send a GET HTTP requests

        Args:
            route (str): String with the route to the endpoint

        Return:
            **response**: Return the response object
        """
        return self._send_request(
            verb="GET",
            url=self._format_url(route),
            params=kwargs,
        ).json()

    def post(self, route, body):
        """ Send a POST HTTP requests

        Args:
            route (str): String with the route to the endpoint
            body (dict): Dict with the body of the request to send

        Return:
            **response**: Return the response object
        """
        return self._send_request(
            verb="POST",
            url=self._format_url(route),
            payload=body
        ).json()

    def put(self, route, body):
        """ Send a PUT HTTP requests

        Args:
            route (str): String with the route to the endpoint
            body (dict): Dict with the body of the request to send

        Return:
            **response**: Return the response object
        """
        return self._send_request(
            verb="PUT",
            url=self._format_url(route),
            payload=body
        ).json()

    def delete(self, route):
        """ Send a DELETE HTTP requests

        Args:
            route (str): String with the route to the endpoint
            body (dict): Dict with the body of the request to send

        Return:
            **response**: Return the response object
        """
        return self._send_request(
            verb="DELETE",
            url=self._format_url(route)
        )

    def _format_url(self, path):
        return "{url}{path_prefix}{path}".format(
            url=self.baseurl,
            path_prefix="/api/rest",
            path=path)

    def _send_request(self, verb, url, payload=None, params={}):
        """send the API request using the *requests.request* method

        Args:
            payload (dict)

        Raises:
            OTRSHTTPError:
            ArgumentMissingError

        Returns:
            **requests.Response**: Response received after sending the request.

        .. note::
            Supported HTTP Methods: DELETE, GET, HEAD, PATCH, POST, PUT
        """
        headers = {
            "Accept": "application/json",
            "Content-Type": "application/json",
        }

        json_payload = None
        if payload:
            json_payload = json.dumps(payload)

        logger.info("{verb} {url} \n {body}".format(verb=verb, url=url, body=payload))
        try:
            response = requests.request(verb.upper(),
                                        url,
                                        headers=headers,
                                        data=json_payload,
                                        params=params,
                                        auth=(self.username, self.password))
        except Exception as err:
            raise exceptions.PyOpenCellHTTPException(err)

        if self._check_response_error(response):
            raise exceptions.PyOpenCellAPIException(
                verb=response.request.method,
                url=response.request.url,
                status=response.status_code,
                body=response.content or ""
            )

        return response

    @staticmethod
    def _check_response_error(response):
        """ Check the response of the OpenCell API. Not always the status code match with the real
        status of the response. We found some cases that response has a 200 OK status but in the
        body we find an error message instead of the expected response.

        We found the next cases:
        * Status 200 but in the body:
          - response.content with "errorCode".
          - response.content with "status" equal to "FAIL".
        * Status of error (>= 400)
        """
        if response.status_code >= 400:
            return True

        if response.content:
            r_json = response.json()
            if "errorCode" in r_json or ("status" in r_json and r_json.get('status') == 'FAIL'):
                return True
        return False



# -*- coding: utf-8 -*-
from setuptools import setup, find_packages, Command
import os
import sys

with open('README.md') as f:
    README = f.read()


VERSION = '0.4.2'


########
# Copied from https://github.com/kennethreitz/setup.py
here = os.path.abspath(os.path.dirname(__file__))


class UploadCommand(Command):
    """Support setup.py upload."""

    description = 'Build and publish the package.'
    user_options = []

    @staticmethod
    def status(s):
        """Prints things in bold."""
        print('\033[1m{0}\033[0m'.format(s))

    def initialize_options(self):
        pass

    def finalize_options(self):
        pass

    def run(self):
        self.status('Pushing git tags…')
        os.system('git tag v{0}'.format(VERSION))
        os.system('git push --tags')

        sys.exit()
########


setup(
    name='pyopencell',
    version=VERSION,
    author='Coopdevs',
    author_email='info@coopdevs.org',
    maintainer='Daniel Palomar, Antonio Barcia',
    url='https://gitlab.com/coopdevs/pyopencell',
    description='Python wrapper for OpenCell (using REST API)',
    long_description=README,
    long_description_content_type='text/markdown',
    packages=find_packages(exclude=('tests', 'docs')),
    include_package_data=True,
    zip_safe=False,
    install_requires=['requests'],
    test_suite='unittest2.collector',
    tests_require=['unittest2', 'mock', 'tox', 'coverage'],
    classifiers=[
        # How mature is this project? Common values are
        #   3 - Alpha
        #   4 - Beta
        #   5 - Production/Stable
        'Development Status :: 3 - Alpha',

        'Operating System :: POSIX :: Linux',

        # Specify the Python versions you support here. In particular, ensure
        # that you indicate whether you support Python 2, Python 3 or both.
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.7',
    ],
    # $ setup.py publish support.
    cmdclass={
        'upload': UploadCommand,
    },
)



from pyopencell.client import Client
from pyopencell.resources.base_resource_list import BaseResourceList
from pyopencell.resources.subscription import Subscription
from pyopencell.responses.paged_response import PagedResponse
from pyopencell.responses.action_status import ActionStatus


class SubscriptionList(BaseResourceList):
    _name = "subscriptions"
    _url_path = "/billing/subscription/list"

    items_resource_class = Subscription
    items_per_page = 10

    subscriptions = []

    @classmethod
    def get(cls, query=None, limit=None, offset=None):
        """
        Returns a Subscription list with subscription instances data.

        :return: PagedResponse:
        """
        if not limit:
            limit = cls.items_per_page

        response_data = Client().get(
            cls._url_path,
            query=query,
            limit=limit,
            offset=offset
        )

        status = response_data.get("status")
        if status and status != "SUCCESS":
            return ActionStatus(**response_data)

        return PagedResponse(cls, **response_data)

    @classmethod
    def get_instances_from_response(self, **response_data):
        if "subscriptions" not in response_data:
            return []

        return response_data["subscriptions"]["subscription"]



from pyopencell.client import Client
from pyopencell.resources.base_resource import BaseResource
from pyopencell.responses.action_status import ActionStatus


class CRMAccountHierarchy(BaseResource):
    _url_path = "/account/accountHierarchy"

    crmAccountType = ""
    crmParentCode = ""
    code = ""
    description = ""
    externalRef1 = ""
    externalRef2 = ""
    name = {
        "title": "",
        "firstName": "",
        "lastName": "",
    }
    address = {
        "address1": "",
        "address2": "",
        "address3": "",
        "zipCode": "",
        "city": "",
        "country": "",
        "state": ""
    }
    contactInformation = {
        "email": "",
        "phone": "",
        "mobile": "",
        "fax": ""
    }
    jobTitle = ""
    language = ""
    terminationReason = ""
    subscriptionDate = None
    terminationDate = None
    customerCategory = ""
    customerBrand = ""
    registrationNo = ""
    vatNo = ""
    mandateIdentification = ""
    mandateDate = None
    currency = ""
    # TODO: Create this list of values in the resource CustomerAccount
    caStatus = ""  # Possible value: 'ACTIVE' or 'CLOSE'
    creditCategory = ""
    dateStatus = None
    dateDunningLevel = None
    # TODO: Create this list of values in the resource Dunning if is needed.
    dunningLevel = ""  # Possible value: 'R0', 'R1', 'R2', 'R3' or 'R4'
    methodOfPayment = []
    billingCycle = ""
    country = ""
    nextInvoiceDate = None
    electronicBilling = False
    # TODO: Create this list of values in the resource BillingAccount
    baStatus = ""  # Possibles values: 'ACTIVE', 'CANCELED', 'TERMINATED' or 'CLOSED'
    email = ""
    invoicingThreshold = None
    # TODO: Create this list of values in the resource UserAccount
    uaStatus = ""  # Possibles values: 'ACTIVE', 'CANCELED', 'TERMINATED' or 'CLOSED'
    cfToAdd = {}
    cfMapToAdd = {}
    customFields = []
    discountPlanForInstantiation = []
    discountPlanForTermination = []

    @classmethod
    def create(cls, **kwargs):
        """
        Creates a CRMAccountHierarchy instance.
        This resource in OpenCell is not a single resource, is a hierarchy os resources,
        from Customer to User Account.

        :param kwargs:
        :return:
        """
        response_data = Client().post(
            "{}/{}".format(cls._url_path, 'createCRMAccountHierarchy'),
            body=kwargs)

        return ActionStatus(**response_data)

    @classmethod
    def update(cls, **kwargs):
        """
        Update a CRMAccountHierarchy instance.

        :param kwargs:
        :return:
        """
        response_data = Client().post(
            "{}/{}".format(cls._url_path, 'createOrUpdateCRMAccountHierarchy'),
            body=kwargs)

        return ActionStatus(**response_data)
